/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/

#include "Settings/settings_scene.h"
#include "Settings/settings_widget.h"

#include "threadmanager.h"

#include "button_item.h"
#include "CollectionView/graphicitem.h"

#include "debug.h"

// Qt
#include <QGraphicsProxyWidget>
#include <QtCore>

/*
********************************************************************************
*                                                                              *
*    Class SettingsScene                                                       *
*                                                                              *
********************************************************************************
*/
SettingsScene::SettingsScene(QWidget* parent)
{
    m_parentWidget = parent;
    this->setParent(parent);
    this->setItemIndexMethod(QGraphicsScene::NoIndex);

    m_initialized = false;
}

SettingsScene::~SettingsScene()
{
    clear();
}

/*******************************************************************************
    init
*******************************************************************************/
void SettingsScene::init()
{
    // create pages
    m_pages[SETTINGS::GENERAL]   = new PageGeneral(m_parentWidget);
    m_pages[SETTINGS::PLAYER]    = new PagePlayer(m_parentWidget);
    m_pages[SETTINGS::LIBRARY]   = new PageLibrary(m_parentWidget);
    m_pages[SETTINGS::RADIO]     = new PageRadio(m_parentWidget);
    m_pages[SETTINGS::SHORTCUT]  = new PageShortcut(m_parentWidget);
    m_pages[SETTINGS::SCROBBLER] = new PageScrobbler(m_parentWidget);

    m_header = new HeaderItem();
    m_header->setText( tr("Settings") );

    m_controlWidget = new ControlWidget(m_parentWidget);
    connect(m_controlWidget, SIGNAL(save_clicked()), this, SLOT(slot_apply_settings()));
    connect(m_controlWidget, SIGNAL(cancel_clicked()), this, SLOT(slot_cancel_settings()));

    m_buttons[SETTINGS::GENERAL] = new ButtonStateItem();
    m_buttons[SETTINGS::GENERAL]->setText(tr("General"));
    m_buttons[SETTINGS::GENERAL]->setPixmap(QPixmap(":/icon/64x64/yarock.png"));
    m_buttons[SETTINGS::GENERAL]->setData(SETTINGS::GENERAL);
    m_buttons[SETTINGS::GENERAL]->setChecked(true);

    m_buttons[SETTINGS::PLAYER] = new ButtonStateItem();
    m_buttons[SETTINGS::PLAYER]->setText(tr("Player"));
    m_buttons[SETTINGS::PLAYER]->setPixmap(QPixmap(":/images/media-play.png"));
    m_buttons[SETTINGS::PLAYER]->setData(SETTINGS::PLAYER);

    m_buttons[SETTINGS::LIBRARY] = new ButtonStateItem();
    m_buttons[SETTINGS::LIBRARY]->setText(tr("Library"));
    m_buttons[SETTINGS::LIBRARY]->setPixmap(QPixmap(":/images/folder-music-48x48.png"));
    m_buttons[SETTINGS::LIBRARY]->setData(SETTINGS::LIBRARY);

    m_buttons[SETTINGS::RADIO]   = new ButtonStateItem();
    m_buttons[SETTINGS::RADIO]->setText(tr("Radio"));
    m_buttons[SETTINGS::RADIO]->setPixmap(QPixmap(":/images/media-url-48x48.png"));
    m_buttons[SETTINGS::RADIO]->setData(SETTINGS::RADIO);

    m_buttons[SETTINGS::SHORTCUT] = new ButtonStateItem();
    m_buttons[SETTINGS::SHORTCUT]->setText(tr("Shortcut"));
    m_buttons[SETTINGS::SHORTCUT]->setPixmap(QPixmap(":/images/input-keyboard-48x48.png"));
    m_buttons[SETTINGS::SHORTCUT]->setData(SETTINGS::SHORTCUT);

    m_buttons[SETTINGS::SCROBBLER] = new ButtonStateItem();
    m_buttons[SETTINGS::SCROBBLER]->setText(tr("Scrobbler"));
    m_buttons[SETTINGS::SCROBBLER]->setPixmap(QPixmap(":/images/lastfm.png"));
    m_buttons[SETTINGS::SCROBBLER]->setData(SETTINGS::SCROBBLER);

    foreach(ButtonStateItem* button, m_buttons.values())
      connect(button, SIGNAL(clicked()), this, SLOT(slot_change_page()));

    this->addItem(m_buttons.value(SETTINGS::GENERAL) );
    this->addItem(m_buttons.value(SETTINGS::PLAYER) );
    this->addItem(m_buttons.value(SETTINGS::LIBRARY) );
    this->addItem(m_buttons.value(SETTINGS::RADIO) );
    this->addItem(m_buttons.value(SETTINGS::SHORTCUT) );
    this->addItem(m_buttons.value(SETTINGS::SCROBBLER) );
    this->addItem(m_pages.value(SETTINGS::GENERAL));

    this->addItem(m_controlWidget);
    this->addItem(m_header);


    m_currentPage = SETTINGS::GENERAL;

    /* first init => always restore settings */
    restore_settings();

    m_initialized = true;
    
    /* il faut interdire la modification des parametres de collection si un 
     database builder est en cours */
    connect(ThreadManager::instance(), SIGNAL(dbBuildStart()), this, SLOT(slot_dbBuilder_stateChange()));
    connect(ThreadManager::instance(), SIGNAL(dbBuildFinished()), this, SLOT(slot_dbBuilder_stateChange()));
}


/*******************************************************************************
    slot_dbBuilder_stateChange
*******************************************************************************/
void SettingsScene::slot_dbBuilder_stateChange()
{
    //Debug::debug() << "SettingsScene::slot_dbBuilder_stateChange";
    m_pages[SETTINGS::LIBRARY]->setEnabled( !ThreadManager::instance()->isDbRunning() );
}

/*******************************************************************************
    populate
*******************************************************************************/
void SettingsScene::populate()
{
    Debug::debug() << "SettingsScene::populate";
    
    QGraphicsView* view = qobject_cast<QGraphicsView*> (m_parentWidget);
    int utilWidth = view->verticalScrollBar()->isVisible() ? (view->width() - view->verticalScrollBar()->width())  : view->width();

    
    /* header item */    
    m_header->_width  = utilWidth - 10;
    m_header->setPos(0, 5);

    /* navigation links */    
    int xPos = m_buttons.value(SETTINGS::GENERAL)->width()  + 20 +
               m_buttons.value(SETTINGS::PLAYER)->width()   + 20 +
               m_buttons.value(SETTINGS::LIBRARY)->width()  + 20 +
               m_buttons.value(SETTINGS::RADIO)->width()    + 20 +
               m_buttons.value(SETTINGS::SHORTCUT)->width() + 20 +
               m_buttons.value(SETTINGS::SCROBBLER)->width();

    xPos = (xPos > m_parentWidget->width()) ? 20 : (m_parentWidget->width() - xPos)/2;

    m_buttons.value(SETTINGS::GENERAL)->setPos(xPos, 45);

    xPos+=m_buttons.value(SETTINGS::GENERAL)->width() + 20;
    m_buttons.value(SETTINGS::PLAYER)->setPos(xPos, 45);

    xPos+=m_buttons.value(SETTINGS::PLAYER)->width() + 20;
    m_buttons.value(SETTINGS::LIBRARY)->setPos(xPos, 45);

    xPos+=m_buttons.value(SETTINGS::LIBRARY)->width() + 20;
    m_buttons.value(SETTINGS::RADIO)->setPos(xPos, 45);

    xPos+=m_buttons.value(SETTINGS::RADIO)->width() + 20;
    m_buttons.value(SETTINGS::SHORTCUT)->setPos(xPos, 45);

    xPos+=m_buttons.value(SETTINGS::SHORTCUT)->width() + 20;
    m_buttons.value(SETTINGS::SCROBBLER)->setPos(xPos, 45);


    /* settings pages */ 
    m_pages.value(m_currentPage)->setPos(5,80);

    static_cast<PageGeneral*>(m_pages[SETTINGS::GENERAL])->doLayout();
    static_cast<PagePlayer*>(m_pages[SETTINGS::PLAYER])->doLayout();
    static_cast<PageLibrary*>(m_pages[SETTINGS::LIBRARY])->doLayout();
    static_cast<PageRadio*>(m_pages[SETTINGS::RADIO])->doLayout();
    static_cast<PageShortcut*>(m_pages[SETTINGS::SHORTCUT])->doLayout();
    static_cast<PageScrobbler*>(m_pages[SETTINGS::SCROBBLER])->doLayout();

    int height = 0;
    height = qMax( height, static_cast<PageGeneral*>(m_pages[SETTINGS::GENERAL])->size().height() );
    height = qMax( height, static_cast<PagePlayer*>(m_pages[SETTINGS::PLAYER])->size().height() );
    height = qMax( height, static_cast<PageLibrary*>(m_pages[SETTINGS::LIBRARY])->size().height() );
    height = qMax( height, static_cast<PageRadio*>(m_pages[SETTINGS::RADIO])->size().height() );
    height = qMax( height, static_cast<PageShortcut*>(m_pages[SETTINGS::SHORTCUT])->size().height() );
    height = qMax( height, static_cast<PageScrobbler*>(m_pages[SETTINGS::SCROBBLER])->size().height() );

    /* control widget */
    xPos = 30 + m_controlWidget->size().width() + 30;
    xPos = (xPos > m_parentWidget->width()) ? 30 : (m_parentWidget->width() - xPos)/2;

    m_controlWidget->setPos(xPos, height+95+30);

    
    /* set scene rect */   
    int diff = m_parentWidget->height() - itemsBoundingRect().height();
    int ajust  = ( diff > 0) ? diff : 0;    

    setSceneRect ( itemsBoundingRect().adjusted(-10, -10, 0, ajust-10) );
}


/*******************************************************************************
    slot_change_page
*******************************************************************************/
void SettingsScene::slot_change_page()
{
    //Debug::debug() << "SettingsScene::slot_change_page";

    ButtonStateItem *button = qobject_cast<ButtonStateItem *>(sender());
    if(!button) return;

    foreach(ButtonStateItem* b, m_buttons.values()) {
      if(button != b) {
        b->setChecked( false);
        b->update();
      }
    }

    this->removeItem( m_pages.value(m_currentPage) );
    m_pages.value(m_currentPage)->hide();

    SETTINGS::SETTINGS_PAGES page_type = SETTINGS::SETTINGS_PAGES (button->data().toInt());
    this->addItem( m_pages.value(page_type) );
    m_pages.value(page_type)->setPos(5,80);
    m_pages.value(page_type)->show();

    m_currentPage = page_type;
}

/*******************************************************************************
    set_page
*******************************************************************************/
void SettingsScene::set_page(SETTINGS::SETTINGS_PAGES page)
{
    foreach(ButtonStateItem* b, m_buttons.values()) {
      if(page == SETTINGS::SETTINGS_PAGES (b->data().toInt()))
        b->setChecked(true);
      else
        b->setChecked(false);

      b->update();
    }

    this->removeItem( m_pages.value(m_currentPage) );
    m_pages.value(m_currentPage)->hide();

    this->addItem( m_pages.value(page) );
    m_pages.value(page)->setPos(5,80);
    m_pages.value(page)->show();

    m_currentPage = page;
}

/*******************************************************************************
    slot_cancel_settings
*******************************************************************************/
void SettingsScene::slot_cancel_settings()
{
    //Debug::debug() << "SettingsScene::slot_cancel_settings";
    restore_settings();
}

/*******************************************************************************
    restore_settings
*******************************************************************************/
void SettingsScene::restore_settings()
{
    Debug::debug() << "SettingsScene::restore_settings";
    static_cast<PageGeneral*>(m_pages[SETTINGS::GENERAL])->restoreSettings();
    static_cast<PagePlayer*>(m_pages[SETTINGS::PLAYER])->restoreSettings();
    static_cast<PageLibrary*>(m_pages[SETTINGS::LIBRARY])->restoreSettings();
    static_cast<PageRadio*>(m_pages[SETTINGS::RADIO])->restoreSettings();
    static_cast<PageShortcut*>(m_pages[SETTINGS::SHORTCUT])->restoreSettings();
    static_cast<PageScrobbler*>(m_pages[SETTINGS::SCROBBLER])->restoreSettings();


    m_result.isSystrayChanged       =  false;
    m_result.isDbusChanged          =  false;
    m_result.isMprisChanged         =  false;
    m_result.isCollectionChanged    =  false;
    m_result.isRadioChanged         =  false;
    m_result.isShorcutChanged       =  false;
    m_result.isScrobblerChanged     =  false;
}

/*******************************************************************************
    slot_apply_settings
*******************************************************************************/
void SettingsScene::slot_apply_settings()
{
    Debug::debug() << "SettingsScene::slot_apply_settings";

    //! save setting
    m_result.isSystrayChanged       =  static_cast<PageGeneral*>(m_pages[SETTINGS::GENERAL])->isSystrayChanged();
    m_result.isDbusChanged          =  static_cast<PageGeneral*>(m_pages[SETTINGS::GENERAL])->isDbusChanged();
    m_result.isMprisChanged         =  static_cast<PageGeneral*>(m_pages[SETTINGS::GENERAL])->isMprisChanged();
    m_result.isCollectionChanged    =  static_cast<PageLibrary*>(m_pages[SETTINGS::LIBRARY])->isChanged();
    m_result.isRadioChanged         =  static_cast<PageRadio*>(m_pages[SETTINGS::RADIO])->isChanged();
    m_result.isShorcutChanged       =  static_cast<PageShortcut*>(m_pages[SETTINGS::SHORTCUT])->isChanged();
    m_result.isScrobblerChanged     =  static_cast<PageScrobbler*>(m_pages[SETTINGS::SCROBBLER])->isChanged();

    static_cast<PageGeneral*>(m_pages[SETTINGS::GENERAL])->saveSettings();
    static_cast<PagePlayer*>(m_pages[SETTINGS::PLAYER])->saveSettings();
    static_cast<PageLibrary*>(m_pages[SETTINGS::LIBRARY])->saveSettings();
    static_cast<PageRadio*>(m_pages[SETTINGS::RADIO])->saveSettings();
    static_cast<PageShortcut*>(m_pages[SETTINGS::SHORTCUT])->saveSettings();
    static_cast<PageScrobbler*>(m_pages[SETTINGS::SCROBBLER])->saveSettings();

    emit settings_saved();
}

